@@ -6,6 +6,8 @@
 #'
 #' @details in case \code{e2} is numeric, +, -, *, /, %% and %/% add, subtract, multiply, divide, modulo, or integer-divide by \code{e2}. In case \code{e2} is an n x n matrix, * matrix-multiplies and / multiplies by its inverse. If \code{e2} is an \code{sfg} object, |, /, & and %/% result in the geometric union, difference, intersection and symmetric difference respectively, and \code{==} and \code{!=} return geometric (in)equality, using \link{st_equals}.
 #'
+#' If \code{e1} is of class \code{sfc}, and \code{e2} is a length 2 numeric, then it is considered a two-dimensional point (and if needed repeated as such) only for operations \code{+} and \code{-}, in other cases the individual numbers are repeated; see commented examples.
+#'
 #' @return object of class \code{sfg}
 #' @export
 #'
@@ -26,14 +28,17 @@
 #' mp - st_point(c(3,4)) # POINT (1 2)
 #' opar = par(mfrow = c(2,2), mar = c(0, 0, 1, 0))
 #' a = st_buffer(st_point(c(0,0)), 2)
-#' b = a + c(2, 0) 
+#' b = a + c(2, 0)
 #' p = function(m) { plot(c(a,b)); plot(eval(parse(text=m)), col=grey(.9), add = TRUE); title(m) }
 #' lapply(c('a | b', 'a / b', 'a & b', 'a %/% b'), p)
 #' par(opar)
+#' sfc = st_sfc(st_point(0:1), st_point(2:3))
+#' sfc + c(2,3) # added to EACH geometry
+#' sfc * c(2,3) # first geometry multiplied by 2, second by 3
 Ops.sfg <- function(e1, e2) {
 
 	if (nargs() == 1) {
-		switch (.Generic, 
+		switch (.Generic,
 			"-" = return(e1 * -1.0),
 			"+" = return(e1),
 			stop(paste("unary", .Generic, "not defined for \"sfg\" objects"))
@@ -48,7 +53,7 @@ Ops.sfg <- function(e1, e2) {
 
 	if (!(prd || pm || mod || set || lgcl))
 		stop(paste("operation", .Generic, "not supported for sfg objects"))
-	
+
 	if (st_is_empty(e1))
 		return(e1)
 
@@ -92,14 +97,14 @@ Ops.sfg <- function(e1, e2) {
 			function(x, Mat, Vec) structure(if_pt(x, unclass(x) + conform(Vec, x)), class = class(x))
 		else # mod:
 			function(x, Mat, Vec) structure(if_pt(x, unclass(x) %% conform(Vec, x)), class = class(x))
-		
+
 	if (is.list(e1))
 		rapply(e1, fn, how = "replace", Mat = Mat, Vec = Vec)
 	else
 		fn(e1, Mat, Vec)
 }
 
-conform = function(vec, m) { 
+conform = function(vec, m) {
 	if (is.matrix(m))
 		t(matrix(vec, ncol(m), nrow(m)))
 	else
@@ -117,13 +122,24 @@ Ops.sfc <- function(e1, e2) {
 	if (length(e1) == 0) # empty set
 		return(e1)
 
+	if (is.numeric(e2) && !is.matrix(e2) && length(e2) <= 2 && .Generic %in% c("+", "-")) {
+		if (.Generic == "-")
+			e2 <- -e2
+		return(opp_sfc(e1, as.numeric(e2), 0L, NA_crs_))
+	} else if (.Generic %in% c("*", "/") && is.numeric(e2) && (length(e2) == 1 || is_only_diag(e2))) {
+		if (is.matrix(e2)) e2 <- diag(e2)
+		if (.Generic == "/")
+			e2 <- 1 / e2
+		return(opp_sfc(e1, as.numeric(e2), 1L, NA_crs_))
+	}
 	if ((is.matrix(e2) && ncol(e2) == 2) || (is.numeric(e2) && length(e2) == 2))
 		e1 = st_zm(e1) # drop z and/or m
 
-	if (!is.list(e2))
+	if (!is.list(e2) && ((.Generic %in% c("+", "-") && length(e2) == 2) || is.matrix(e2)))
 		e2 = list(e2)
 
-	ret = switch(.Generic,
+	ret = switch(
+		.Generic,
 		"&"   =  mapply(function(x, y) { x  & y }, e1, e2, SIMPLIFY = FALSE),
 		"|"   =  mapply(function(x, y) { x  | y }, e1, e2, SIMPLIFY = FALSE),
 		"%/%" =  mapply(function(x, y) { x %/% y}, e1, e2, SIMPLIFY = FALSE),
@@ -138,10 +154,13 @@ Ops.sfc <- function(e1, e2) {
 
 	if (!(.Generic %in% c("!=", "=="))) {
 		crs = if (.Generic %in% c("&", "|", "%/%", "/") && inherits(e2, c("sfc", "sfg"))) # retain:
-				st_crs(e1)
-			else # geometry got displaced:
-				NA_crs_
+			st_crs(e1)
+		else # geometry got displaced:
+			NA_crs_
 		st_sfc(ret, crs = crs, precision = attr(e1, "precision"))
 	} else
 		ret
 }
+is_only_diag <- function(x) {
+	is.matrix(x) && all(`diag<-`(x, 0) == 0)
+}
