@@ -123,6 +123,12 @@ st_cast_sfc_default = function(x) {
 	structure(st_sfc(x), ids = ids)
 }
 
+copy_sfc_attributes_from = function(x, ret) {
+	structure(ret, precision = attr(x, "precision"),
+		bbox = attr(x, "bbox"), crs = attr(x, "crs"), n_empty = attr(x, "n_empty"))
+}
+
+
 #' @name st_cast
 #' @param ids integer vector, denoting how geometries should be grouped (default: no grouping)
 #' @param group_or_split logical; if TRUE, group or split geometries; if FALSE, carry out a 1-1 per-geometry conversion.
@@ -150,7 +156,7 @@ st_cast.sfc = function(x, to, ..., ids = seq_along(x), group_or_split = TRUE) {
 			st_cast(st_cast(x, "MULTIPOINT"), "POINT")
 		else if (to == "MULTIPOINT") {
 			ret = lapply(x, function(y) structure(as.matrix(y), class = c(class(y)[1], to, "sfg")))
-			attributes(ret) = attributes(x)
+			ret = copy_sfc_attributes_from(x, ret)
 			reclass(ret, to, FALSE)
 		} else
 			#st_cast(st_cast(x, "MULTILINESTRING"), to)
@@ -160,13 +166,13 @@ st_cast.sfc = function(x, to, ..., ids = seq_along(x), group_or_split = TRUE) {
 			lapply(unname(split(x, ids)), function(y) structure(do.call(rbind, y), class = class(x[[1]])))
 		else
 			lapply(unname(split(x, ids)), function(y) structure(y, class = class(x[[1]])))
-		attributes(ret) = attributes(x)
+		ret = copy_sfc_attributes_from(x, ret)
 		reclass(ret, to, need_close(to))
 	} else if (from_col == 3 && to == "MULTILINESTRING") {
 		ret = lapply(x, unlist, recursive = FALSE) # unlist one level deeper; one MULTIPOLYGON -> one MULTILINESTRING
 		if (length(ret))
 			class(ret[[1]]) = class(x[[1]]) # got dropped
-		attributes(ret) = attributes(x) # bbox, crs
+		ret = copy_sfc_attributes_from(x, ret)
 		structure(reclass(ret, to, FALSE))
 	} else { # "horizontal", to the left: split
 		ret = if (from_col == 1) # LINESTRING or MULTIPOINT to POINT
@@ -174,7 +180,7 @@ st_cast.sfc = function(x, to, ..., ids = seq_along(x), group_or_split = TRUE) {
 			else
 				unlist(x, recursive = FALSE)
 		ret = lapply(ret, function(y) structure(y, class = class(x[[1]]))) # will be reset by reclass()
-		attributes(ret) = attributes(x)
+		ret = copy_sfc_attributes_from(x, ret)
 		structure(reclass(ret, to, need_close(to)), ids = get_lengths(x))
 	}
 }
