@@ -956,10 +956,10 @@ Rcpp::List CPL_nary_difference(Rcpp::List sfc) {
 					contained = chk_(GEOSContains_r(hGEOSCtxt, out[tree_sel[j]].get(), geom.get()));
 					if (contained)
 						break;
-					// test if the items overlap with geom
-					if (chk_(GEOSOverlaps_r(hGEOSCtxt, geom.get(), out[tree_sel[j]].get()))) {
+					// test if the items intersect with geom
+					if (chk_(GEOSIntersects_r(hGEOSCtxt, geom.get(), out[tree_sel[j]].get()))) {
 						// if they do then erase overlapping parts from geom
-						geom = std::move(geos_ptr(GEOSDifference_r(hGEOSCtxt, geom.get(), out[tree_sel[j]].get()), hGEOSCtxt));
+						geom = geos_ptr(GEOSDifference_r(hGEOSCtxt, geom.get(), out[tree_sel[j]].get()), hGEOSCtxt);
 						if (geom == nullptr)
 							Rcpp::stop("GEOS exception"); // #nocov
 						// ensure that geom is valid
@@ -1021,7 +1021,7 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 					if (inters == nullptr)
 						Rcpp::stop("GEOS exception"); // #nocov
 					if (! chk_(GEOSisEmpty_r(hGEOSCtxt, inters.get()))) { // i and k intersection
-						geom = std::move(geos_ptr(GEOSDifference_r(hGEOSCtxt, geom.get(), inters.get()), hGEOSCtxt)); // cut out inters from geom
+						geom = geos_ptr(GEOSDifference_r(hGEOSCtxt, geom.get(), inters.get()), hGEOSCtxt); // cut out inters from geom
 						if (geom == nullptr)
 							Rcpp::stop("GEOS exception"); // #nocov
 						GeomPtr g = geos_ptr(GEOSDifference_r(hGEOSCtxt, out[k].get(), inters.get()), hGEOSCtxt); // cut out inters from out[k]
@@ -1047,11 +1047,14 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 	} // for i
 	size_t j = 0;
 	for (size_t i = 0; i < out.size(); i++) {
-	if (! GEOSisEmpty_r(hGEOSCtxt, out[i].get()))
-		out[j] = std::move(out[i]);
-		index[j] = index[i];
-		std::sort(index[j].begin(), index[j].end());
-		j++;
+		if (! GEOSisEmpty_r(hGEOSCtxt, out[i].get())) {
+			if (i != j) {
+				out[j] = std::move(out[i]);
+				index[j] = index[i];
+			}
+			std::sort(index[j].begin(), index[j].end());
+			j++;
+		}
 	}
 	out.resize(j);
 	index.resize(j);
