@@ -104,7 +104,7 @@ Rcpp::List CPL_polygonize(Rcpp::CharacterVector raster, Rcpp::CharacterVector ma
 			if (GDALFPolygonize((GDALRasterBandH) poBand, maskBand,
 				(OGRLayerH) poLayer,
 				iPixValField[0],
-				NULL, // create_options(options, true),
+				create_options(options, true).data(),
 				NULL, NULL) != OGRERR_NONE)
 					Rcpp::Rcout << "GDALFPolygonize returned an error" << std::endl; // #nocov
 		} else {
@@ -120,7 +120,8 @@ Rcpp::List CPL_polygonize(Rcpp::CharacterVector raster, Rcpp::CharacterVector ma
 
 	Rcpp::NumericVector type(1);
 	type[0] = 0;
-	Rcpp::List lst = sf_from_ogrlayer(poLayer, false, true, type, true);
+	Rcpp::CharacterVector fid_column; // empty
+	Rcpp::List lst = sf_from_ogrlayer(poLayer, false, true, type, fid_column, true);
 	GDALClose(poDataset); // raster
 	GDALClose(poDS); // vector
 	if (maskDataset != NULL)
@@ -158,6 +159,10 @@ Rcpp::List CPL_rasterize(Rcpp::CharacterVector raster, Rcpp::CharacterVector ras
 		NULL, // GDALProgressFunc 	pfnProgress,
 		NULL  //void * 	pProgressArg 
 	);
+
+	for (size_t i = 0; i < geoms.size(); i++)
+		OGRGeometryFactory::destroyGeometry(geoms[i]);
+
 	if (err != OGRERR_NONE)
 		Rcpp::Rcout << "GDALRasterizeGeometries returned an error" << std::endl; // #nocov
 	GDALClose(poDataset); // raster
