@@ -96,3 +96,10 @@ storms.sf = st_as_sf(storms, coords = c("long", "lat"), crs = 4326)
 x <- storms.sf %>% group_by(name, year) %>% nest
 
 nrow(distinct(nc[c(1,1,1,2,2,3:100),]))
+
+set.seed(1331)
+nc$gp <- sample(1:10, replace=T)
+# Get centroid of each group of polygons; https://github.com/r-spatial/sf/issues/969
+nc_gp_cent <- nc %>%
+                group_by(gp) %>%
+                group_map(st_centroid)
